'use server'

import { requestGetFetch, requestPostFetch } from "@/utils/fetch";
import { getServerSession, Session } from "next-auth";
import { nextauthOptions } from "../nextauth-options";
import { SearchParams } from "@/types/searchParams";

export async function getRoles(searchParams?: SearchParams) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;

    const paramsArray = searchParams && Object.entries(searchParams);

    const queryString = paramsArray && paramsArray.map(([key, value]) => `${key}=${value}`).join('&');

    const path = 'v1/roles' + `${queryString ? `?${queryString}` : ''}`;

    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });
    return response.data || [];
  } catch (error) {
    throw error;
  }
}

export async function getPermissions() {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = "v1/model-by-permission";
    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });
    return response.data || [];
  } catch (error) {
    throw error;
  }
}

export async function getPermissionsById(id: number) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/roles/${id}`;
    const response = await requestGetFetch({
      path,
      options: {
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: "no-store",
      },
    });
    return response.data || [];
  } catch (error) {
    throw error;
  }
}


export async function deleteRole(id: number) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/roles/${id}`;
    const response = await requestGetFetch({
      path,
      options: {
        method: "DELETE",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        cache: 'no-store',
      },
    });
    return response;
  } catch (error) {
    throw error;
  }
}

interface CreateRoleProps {
  name: string;
  projects: number[];
  permissions: string[]; 
}

export async function createRole({
  name,
  projects,
  permissions,
}: CreateRoleProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const response = await requestPostFetch({
      path: `v1/roles`,
      data: {
        name,
        projects,
        permissions,
      },
      token: token,
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}


interface updateRoleProps {
  id: number;
  name: string;
  projects: number[];
  permissions: string[];
}

export async function updateRole({
  id,
  name,
  projects,
  permissions,
}: updateRoleProps) {
  try {
    const session: Session = await getServerSession(nextauthOptions);
    const token = session?.user.token;
    const path = `v1/roles/${id}`;
    
    const response = await requestGetFetch({
      path,
      options: {
        method: "PUT",
        headers: {
          Authorization: `Bearer ${token}`,
          "Content-Type": "application/json",
          Accept: "application/json",
        },
        body: JSON.stringify({
          name,
          projects,
          permissions,
        }), 
        cache: 'no-store',
      },
    });
    return response;
  } catch (error: any) {
    throw new Error(error.message);
  }
}
